﻿using Infragistics.Windows.DataPresenter;
using System.Collections;
using System.Collections.Specialized;
using System.Windows;
using System.Windows.Interactivity;

namespace Bill.Helpers
{
    public class XamDataGridSelectedItemsBehavior : Behavior<XamDataGrid>
    {
        // Source: The XamDataGrid's SelectedRows collection
        // Target: The object bound as the behaviors SelectedItems collection

        private XamDataGrid Grid
        {
            get
            {
                return AssociatedObject as XamDataGrid;
            }
        }

        public INotifyCollectionChanged SelectedItems
        {
            get { return (INotifyCollectionChanged)GetValue(SelectedItemsProperty); }
            set { SetValue(SelectedItemsProperty, value); }
        }

        public static readonly DependencyProperty SelectedItemsProperty = DependencyProperty.Register(
            "SelectedItems",
            typeof(INotifyCollectionChanged),
            typeof(XamDataGridSelectedItemsBehavior),
            null);

        protected override void OnAttached()
        {
            base.OnAttached();

            Grid.SelectedItemsChanged += new System.EventHandler<Infragistics.Windows.DataPresenter.Events.SelectedItemsChangedEventArgs>(Grid_SelectedItemsChanged);

            if (SelectedItemsProperty.GetMetadata(typeof(XamDataGridSelectedItemsBehavior)).PropertyChangedCallback == null)
                SelectedItemsProperty.OverrideMetadata(typeof(XamDataGridSelectedItemsBehavior), new FrameworkPropertyMetadata(OnSelectedItemsChanged));

             //SelectedItems.CollectionChanged += new NotifyCollectionChangedEventHandler(SelectedItems_CollectionChanged);
        }

        private void OnSelectedItemsChanged(DependencyObject obj, DependencyPropertyChangedEventArgs e)
        {
            if (e.OldValue != null)
                (e.OldValue as INotifyCollectionChanged).CollectionChanged -= SelectedItems_CollectionChanged;

            if (e.NewValue != null)
                (e.NewValue as INotifyCollectionChanged).CollectionChanged += SelectedItems_CollectionChanged;
        }

        void SelectedItems_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            UnsubscribeFromEvents();

            TransferTargetToSource(this.SelectedItems as IList, Grid);

            SubscribeToEvents();
        }

        void Grid_SelectedItemsChanged(object sender, Infragistics.Windows.DataPresenter.Events.SelectedItemsChangedEventArgs e)
        {
            UnsubscribeFromEvents();

            TransferSourceToTarget(Grid.SelectedItems.Records, this.SelectedItems as IList);

            SubscribeToEvents();
        }

        private void SubscribeToEvents()
        {
            Grid.SelectedItemsChanged += new System.EventHandler<Infragistics.Windows.DataPresenter.Events.SelectedItemsChangedEventArgs>(Grid_SelectedItemsChanged);
            if (SelectedItems != null)
                SelectedItems.CollectionChanged += SelectedItems_CollectionChanged;
        }

        private void UnsubscribeFromEvents()
        {
            Grid.SelectedItemsChanged -= new System.EventHandler<Infragistics.Windows.DataPresenter.Events.SelectedItemsChangedEventArgs>(Grid_SelectedItemsChanged);
            if (SelectedItems != null)
                SelectedItems.CollectionChanged -= SelectedItems_CollectionChanged;
        }

        public static void TransferSourceToTarget(SelectedItemCollectionBase source, IList target)
        {
            if (source == null || target == null)
                return;

            target.Clear();

            foreach (Record r in source)
            {
                // make sure we only take data rows from the root band
                if (r.IsDataRecord && r.ParentRecord == null)
                {
                    target.Add((r as DataRecord).DataItem);
                }
            }
        }

        public static void TransferTargetToSource(IList target, XamDataGrid source)
        {
            if (source == null || target == null)
                return;

            source.SelectedItems.Records.Clear();

            foreach (object obj in target)
            {
                foreach (Record record in source.Records)
                {
                    if ((record as DataRecord).DataItem == obj)
                    {
                        source.SelectedItems.Records.Add(record);
                        (record as DataRecord).IsSelected=true;
                    }
                }
            }

            // If selected items is still empty then we don't have an active record anymore.
            if (source.SelectedItems.Records.Count == 0)
                source.ActiveRecord = null;
        }
    }
}